#!/bin/bash

echo Drop tables ...
perl -Ilib scripts/drop.tables.pl
echo Create tables ...
perl -Ilib scripts/create.tables.pl
echo Populate tables ...
perl -Ilib scripts/populate.tables.pl
echo Populate fake data ...
perl -Ilib scripts/populate.fake.data.pl
