<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN"
"http://www.w3.org/TR/html4/strict.dtd">
<html>

<head>

<title>Contacts</title>
<meta http-equiv="Content-Type" content="text/html; charset=utf-8">

<style type="text/css" title="currentStyle">
	@import "/assets/js/DataTables-1.9.4/media/css/demo_page.css";
	@import "/assets/js/DataTables-1.9.4/media/css/demo_table.css";
</style>

<link rel="stylesheet" type="text/css" href="/assets/css/app/office/contacts/homepage.css">

</head>

<body>

<h3 class="blue_centered">FAQ for Contacts</h3>

<table class="display" id="occ_table_div" cellpadding="0" cellspacing="0" border="0">
<thead>
<tr>
	<th align="left">Item</th>
	<th align="left">Explanation</th>
</tr>
</thead>
<tbody>

<tr class="odd gradeC">
<td>
Adding staff
</td>
<td>
You must use the Add Person tab to add a person to the database<br>
before adding them as staff of an organization.
</td>
</tr>

<tr class="even gradeC">
<td>
Adding occupations
</td>
<td>
Normally, you must use the Add Organization tab to add an organization to the database<br>
before adding an occupation for someone at that organization.<br>
You can however always use the dummy organization '-' (without the quotes) if desired.<br>
Likewise there is a dummy occupation, also called '-', available if required.
</td>
</tr>

<tr class="odd gradeC">
<td>
Deleting people<br>or organizations
</td>
<td>
You can either set their Visibility to no-one, so that they stay in the database<br>
or you can click the Delete button, and remove all references to them. See also the next point.
</td>
</tr>

<tr class="even gradeC">
<td>
What is this Visibility thing?
</td>
<td>
Visibility is the difference between public and private contacts (when people share databases). Values:<br>
o Everyone: Such contacts will appear in other people's search results.<br>
o Just me: Such contacts only appear in your search results.<br>
o No-one: These contacts do not appear in anyone's search results.<br>
</td>
</tr>

<tr class="odd gradeC">
<td>
Which fields can I use autocomplete on?
</td>
<td>
When entering staff, use autocomplete for Person name and Occupation title.<br>
When entering occupations, use autocomplete for Organization name and Occupation title.<br>
You need to type at least 2 characters to trigger the autocomplete code.
</td>
</tr>

<tr class="even gradeC">
<td>
What do I do about the 'Person already on file'<br>
error when adding a new person?
</td>
<td>
Simplest is to change their Preferred name, so that the combination of<br>
'Preferred name + Surname' does not clash with that of the other person.
</td>
</tr>

</tbody>
<tfoot>
<tr>
	<th align="left">Item</th>
	<th align="left">Explanation</th>
</tr>
</tfoot>
</table>

</body>
</html>
